using System;
using System.Windows;
using System.Windows.Input;
using System.Windows.Threading;

namespace BindingToLinqToSql
{
    public class DelegateCommand<T> : ICommand
    {
        private readonly Action<T> executeMethod;
        private readonly Func<T, bool> canExecuteMethod;
        private readonly Dispatcher dispatcher;

        public DelegateCommand(Action<T> executeMethod)
            : this(executeMethod, null)
        {
        }

        public DelegateCommand(Action<T> executeMethod, Func<T, bool> canExecuteMethod)
        {
            if (executeMethod == null && canExecuteMethod == null)
                throw new ArgumentNullException("executeMethod");

            this.executeMethod = executeMethod;
            this.canExecuteMethod = canExecuteMethod;
            if (Application.Current != null)
            {
                this.dispatcher = Application.Current.Dispatcher;
            }
        }

        public bool CanExecute(T parameter)
        {
            if (this.canExecuteMethod == null) return true;
            return this.canExecuteMethod(parameter);
        }

        public void Execute(T parameter)
        {
            if (this.executeMethod == null) return;
            this.executeMethod(parameter);
        }

        bool ICommand.CanExecute(object parameter)
        {
            return this.CanExecute((T)parameter);
        }

        public event EventHandler CanExecuteChanged;

        void ICommand.Execute(object parameter)
        {
            this.Execute((T)parameter);
        }

        public void RaiseCanExecuteChanged()
        {
            this.OnCanExecuteChanged();
        }

        protected virtual void OnCanExecuteChanged()
        {
            EventHandler canExecuteChangedHandler = this.CanExecuteChanged;
            if (canExecuteChangedHandler != null)
            {
                if (this.dispatcher != null && !this.dispatcher.CheckAccess())
                {
                    this.dispatcher.BeginInvoke(DispatcherPriority.Normal,
                                           (Action)this.OnCanExecuteChanged);
                }
                else
                {
                    canExecuteChangedHandler(this, EventArgs.Empty);
                }
            }
        }
    }
}